@@ -1,11 +1,12 @@
 from typing import Optional, List, Union
 
 import torch
+import torch.fx
 
 import e3nn
 from e3nn import o3
-from e3nn.util.codegen import CodeGenMixin
 from e3nn.util.jit import compile_mode
+from e3nn.util.codegen import CodeGenMixin
 from e3nn.util import prod
 
 from ._instruction import Instruction
@@ -189,22 +190,6 @@ class TensorProduct(CodeGenMixin, torch.nn.Module):
         # === Setup ===
         super().__init__()
 
-        assert normalization in ['component', 'norm'], normalization
-        self.normalization = normalization
-
-        if shared_weights is False and internal_weights is None:
-            internal_weights = False
-
-        if shared_weights is None:
-            shared_weights = True
-
-        if internal_weights is None:
-            internal_weights = True
-
-        assert shared_weights or not internal_weights
-        self.internal_weights = internal_weights
-        self.shared_weights = shared_weights
-
         # Determine irreps
         self.irreps_in1 = o3.Irreps(irreps_in1)
         self.irreps_in2 = o3.Irreps(irreps_in2)
@@ -249,13 +234,29 @@ class TensorProduct(CodeGenMixin, torch.nn.Module):
         ]
         self.instructions = instructions
 
+        assert normalization in ['component', 'norm'], normalization
+        self.normalization = normalization
+
+        if shared_weights is False and internal_weights is None:
+            internal_weights = False
+
+        if shared_weights is None:
+            shared_weights = True
+
+        if internal_weights is None:
+            internal_weights = shared_weights and any(i.has_weight for i in self.instructions)
+
+        assert shared_weights or not internal_weights
+        self.internal_weights = internal_weights
+        self.shared_weights = shared_weights
+
         opt_defaults = e3nn.get_optimization_defaults()
         self._specialized_code = _specialized_code if _specialized_code is not None else opt_defaults['specialized_code']
         self._optimize_einsums = _optimize_einsums if _optimize_einsums is not None else opt_defaults['optimize_einsums']
         del opt_defaults
 
         # Generate the actual tensor product code
-        code_out, code_right, wigners = codegen_tensor_product(
+        graph_out, graph_right, wigners = codegen_tensor_product(
             self.irreps_in1,
             self.in1_var,
             self.irreps_in2,
@@ -268,11 +269,11 @@ class TensorProduct(CodeGenMixin, torch.nn.Module):
             self._specialized_code,
             self._optimize_einsums
         )
-
         self._codegen_register({
-            '_compiled_main_out': code_out,
-            '_compiled_main_right': code_right,
+            "_compiled_main_out": graph_out,
+            "_compiled_main_right": graph_right
         })
+
         self._wigners = wigners
 
         # === Determine weights ===
@@ -744,6 +745,7 @@ class ElementwiseTensorProduct(TensorProduct):
         z_u = x_u \otimes y_u
 
     where :math:`u` runs over the irreps. Note that there are no weights.
+    The output representation is determined by the two input representations.
 
     Parameters
     ----------
@@ -754,10 +756,18 @@ class ElementwiseTensorProduct(TensorProduct):
         representation of the second input
 
     filter_ir_out : iterator of `Irrep`, optional
-        representations of the output
+        filter to select only specific `Irrep` of the output
 
     normalization : {'component', 'norm'}
         see `TensorProduct`
+
+    Examples
+    --------
+    Elementwise scalar product
+
+    >>> ElementwiseTensorProduct("5x1o + 5x1e", "10x1e", ["0e", "0o"])
+    ElementwiseTensorProduct(5x1o+5x1e x 10x1e -> 5x0o+5x0e | 10 paths | 0 weights)
+
     """
     def __init__(
         self,
@@ -818,6 +828,7 @@ class FullTensorProduct(TensorProduct):
         z_{uv} = x_u \otimes y_v
 
     where :math:`u` and :math:`v` run over the irreps. Note that there are no weights.
+    The output representation is determined by the two input representations.
 
     Parameters
     ----------
@@ -828,7 +839,7 @@ class FullTensorProduct(TensorProduct):
         representation of the second input
 
     filter_ir_out : iterator of `Irrep`, optional
-        representations of the output
+        filter to select only specific `Irrep` of the output
 
     normalization : {'component', 'norm'}
         see `TensorProduct`
